@@ -8,9 +8,7 @@ import torch.nn.functional as F
 
 # structs
 
-SelfAttentionOutput = namedtuple('SelfAttentionOutput', ['out', 'mem', 'cmem', 'aux_loss'])
-
-TransformerOutput = namedtuple('TransformerOutput', ['out', 'mem', 'cmem', 'aux_loss'])
+Memory = namedtuple('Memory', ['mem', 'cmem'])
 
 # helper functions
 
@@ -101,21 +99,39 @@ class ConvCompress(nn.Module):
 
 # feedforward
 
+class GELU_(nn.Module):
+    def forward(self, x):
+        return 0.5 * x * (1 + torch.tanh(math.sqrt(2 / math.pi) * (x + 0.044715 * torch.pow(x, 3))))
+
+GELU = nn.GELU if hasattr(nn, 'GELU') else GELU_
+
 class FeedForward(nn.Module):
-    def __init__(self, dim, mult = 4):
+    def __init__(self, dim, mult = 4, dropout = 0., activation = None, glu = False):
         super().__init__()
-        self.net = nn.Sequential(
-            nn.Linear(dim, dim * mult),
-            nn.LeakyReLU(inplace = True),
-            nn.Linear(dim * mult, dim)
-        )
+        activation = default(activation, GELU)
+
+        self.glu = glu
+        self.w1 = nn.Linear(dim, dim * mult * (2 if glu else 1))
+        self.act = activation()
+        self.dropout = nn.Dropout(dropout)
+        self.w2 = nn.Linear(dim * mult, dim)
+
     def forward(self, x, **kwargs):
-        return self.net(x)
+        if not self.glu:
+            x = self.w1(x)
+            x = self.act(x)
+        else:
+            x, v = self.w1(x).chunk(2, dim=-1)
+            x = self.act(x) * v
+
+        x = self.dropout(x)
+        x = self.w2(x)
+        return x
 
 # attention.
 
 class SelfAttention(nn.Module):
-    def __init__(self, dim, seq_len, mem_len, cmem_len, cmem_ratio = 4, heads = 8):
+    def __init__(self, dim, seq_len, mem_len, cmem_len, cmem_ratio = 4, heads = 8, attn_dropout = 0., dropout = 0.):
         super().__init__()
         self.heads = heads
         self.dim_head = dim // heads
@@ -131,9 +147,16 @@ class SelfAttention(nn.Module):
         self.to_kv = nn.Linear(dim, dim * 2, bias = False)
         self.to_out = nn.Linear(dim, dim)
 
-    def forward(self, x, mem = None, cmem = None, pos_emb = None, **kwargs):
+        self.attn_dropout = nn.Dropout(attn_dropout)
+        self.dropout = nn.Dropout(dropout)
+
+    def forward(self, x, memories = None, pos_emb = None, input_mask = None, **kwargs):
         b, t, e, h, dim_h = *x.shape, self.heads, self.dim_head
 
+        mem = cmem = None
+        if memories is not None:
+            mem, cmem = memories
+
         mem = default(mem, lambda: torch.empty(b, 0, e))
         cmem = default(cmem, lambda: torch.empty(b, 0, e))
 
@@ -156,14 +179,21 @@ class SelfAttention(nn.Module):
             pos_dots = shift(pos_dots)
             dots = dots + pos_dots
 
+        if input_mask is not None:
+            mask = input_mask[:, None, :, None] * input_mask[:, None, None, :]
+            mask = F.pad(mask, (mem_len + cmem_len, 0), value = False)
+            dots.masked_fill_(~mask, float('-inf'))
+
         mask = torch.ones(t, kv_len).triu_(diagonal = 1 + kv_len).bool()
         dots.masked_fill_(mask[None, None, ...], float('-inf'))
 
         attn = dots.softmax(dim=-1)
+        attn = self.attn_dropout(attn)
 
         out = torch.einsum('bhij,bhjd->bhid', attn, v)
         out = out.transpose(1, 2).reshape(b, t, -1)
         logits = self.to_out(out)
+        logits = self.dropout(logits)
 
         new_mem = mem
         new_cmem = cmem
@@ -200,12 +230,13 @@ class SelfAttention(nn.Module):
                     full_attn(q, cmem_k, cmem_v)
                 )
 
-        return SelfAttentionOutput(out = logits, mem = new_mem, cmem = new_cmem, aux_loss = aux_loss)
+
+        return logits, Memory(mem = new_mem, cmem = new_cmem), aux_loss
 
 # transformer
 
 class CompressiveTransformer(nn.Module):
-    def __init__(self, num_tokens, dim, seq_len, depth, mem_len = None, cmem_len = None, cmem_ratio = 4, heads = 8, gru_gated_residual = True):
+    def __init__(self, num_tokens, dim, seq_len, depth, mem_len = None, cmem_len = None, cmem_ratio = 4, heads = 8, gru_gated_residual = True, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0.):
         super().__init__()
         mem_len = default(mem_len, seq_len)
         cmem_len = default(cmem_len, mem_len // cmem_ratio)
@@ -220,26 +251,31 @@ class CompressiveTransformer(nn.Module):
 
         wrapper = partial(GRUGating, dim) if gru_gated_residual else Residual
 
-        self.attn_layers = nn.ModuleList([wrapper(PreNorm(dim, SelfAttention(dim, seq_len, mem_len, cmem_len, cmem_ratio, heads))) for _ in range(depth)])
-        self.ff_layers = nn.ModuleList([wrapper(PreNorm(dim, FeedForward(dim))) for _ in range(depth)])
+        self.attn_layers = nn.ModuleList([wrapper(PreNorm(dim, SelfAttention(dim, seq_len, mem_len, cmem_len, cmem_ratio, heads, dropout = attn_layer_dropout, attn_dropout = attn_dropout))) for _ in range(depth)])
+        self.ff_layers = nn.ModuleList([wrapper(PreNorm(dim, FeedForward(dim, dropout = ff_dropout))) for _ in range(depth)])
 
-    def forward(self, x, mem = None, cmem = None):
+    def forward(self, x, memories = None, mask = None):
         x = self.token_emb(x)
         b, t, d = x.shape
 
+        mem = cmem = None
+        if memories is not None:
+            mem, cmem = memories
+
         mem = default(mem, lambda: torch.empty(self.depth, b, 0, d))
         cmem = default(cmem, lambda: torch.empty(self.depth, b, 0, d))
 
-        total_len = mem.shape[2] + cmem.shape[2]
-        pos_emb = self.pos_emb[:, (self.seq_len - t):(t + total_len)]
+        total_len = mem.shape[2] + cmem.shape[2] + t
+        pos_emb = self.pos_emb[:, (self.seq_len - t):total_len]
 
         next_mem = []
         next_cmem = []
         aux_loss = torch.zeros(1, requires_grad = True, **to(x))
 
         for attn, ff, m, c in zip(self.attn_layers, self.ff_layers, mem, cmem):
-            x, mem_out, cmem_out, layer_aux_loss = attn(x, mem = m, cmem = c, pos_emb = pos_emb)
+            x, (mem_out, cmem_out), layer_aux_loss = attn(x, memories = (m, c), input_mask = mask, pos_emb = pos_emb)
             x, = ff(x)
+
             next_mem.append(mem_out)
             next_cmem.append(cmem_out)
             aux_loss = aux_loss + layer_aux_loss
@@ -248,4 +284,4 @@ class CompressiveTransformer(nn.Module):
 
         next_mem, next_cmem = map(torch.stack, (next_mem, next_cmem))
         next_mem, next_cmem = map(lambda x: x.detach(), (next_mem, next_cmem))
-        return TransformerOutput(out = out, mem = next_mem, cmem = next_cmem, aux_loss = aux_loss)
+        return out, Memory(mem = next_mem, cmem = next_cmem), aux_loss
